﻿' Fig. 5.12: GradeBook.vb
' GradeBook class that solves class-average problem using
' sentinel-controlled repetition.
Public Class GradeBook
   Private courseNameValue As String ' course name for this GradeBook

   ' constructor initializes CourseName with String supplied as argument
   Public Sub New(ByVal name As String)
      CourseName = name ' validate and store course name
   End Sub ' New

   ' property that gets and sets the course name; the Set accessor
   ' ensures that the course name has at most 25 characters
   Public Property CourseName() As String
      Get ' retrieve courseNameValue
         Return courseNameValue
      End Get

      Set(ByVal value As String) ' set courseNameValue
         If value.Length <= 25 Then ' if value has 25 or fewer characters
            courseNameValue = value ' store the course name in the object
         Else ' if name has more than 25 characters
            ' set courseNameValue to first 25 characters of parameter name
            ' start at 0, length of 25
            courseNameValue = value.Substring(0, 25)

            Console.WriteLine( _
               "Course name (" & value & ") exceeds maximum length (25).")
            Console.WriteLine( _
               "Limiting course name to first 25 characters." & vbCrLf)
         End If
      End Set
   End Property ' CourseName

   ' display a welcome message to the GradeBook user
   Public Sub DisplayMessage()
      ' this statement uses property CourseName to get the
      ' name of the course this GradeBook represents
      Console.WriteLine("Welcome to the grade book for " _
          & vbCrLf & CourseName & "!" & vbCrLf)
   End Sub ' DisplayMessage

   ' determine class average based on 10 grades entered by user
   Public Sub DetermineClassAverage()
      Dim total As Integer ' sum of grades entered by user
      Dim gradeCounter As Integer ' number of grades input
      Dim grade As Integer ' grade input by user
      Dim average As Double ' average of all grades

      ' initialization phase
      total = 0 ' clear total
      gradeCounter = 0 ' prepare to loop

      ' processing phase
      ' prompt for input and read grade from user
      Console.Write("Enter grade or -1 to quit: ")
      grade = Console.ReadLine()

      ' sentinel-controlled loop where -1 is the sentinel value
      While grade <> -1
         total += grade ' add grade to total
         gradeCounter += 1 ' add 1 to gradeCounter

         ' prompt for and input next grade from user
         Console.Write("Enter grade or -1 to quit: ") ' prompt
         grade = Console.ReadLine() ' input next grade
      End While

      ' termination phase
      If gradeCounter <> 0 Then ' if user entered at least one grade
         ' calculate average of all grades entered
         average = total / gradeCounter

         ' display total and average (with two digits of precision)
         Console.WriteLine(vbCrLf & "Total of the " & gradeCounter & _
            " grades entered is " & total)
         Console.WriteLine("Class average is {0:F}", average)
      Else ' no grades were entered, so output appropriate message
         Console.WriteLine("No grades were entered")
      End If
   End Sub ' DetermineClassAverage
End Class ' GradeBook

' **************************************************************************
' * (C) Copyright 1992-2006 by Deitel & Associates, Inc. and               *
' * Pearson Education, Inc. All Rights Reserved.                           *
' *                                                                        *
' * DISCLAIMER: The authors and publisher of this book have used their     *
' * best efforts in preparing the book. These efforts include the          *
' * development, research, and testing of the theories and programs        *
' * to determine their effectiveness. The authors and publisher make       *
' * no warranty of any kind, expressed or implied, with regard to these    *
' * programs or to the documentation contained in these books. The authors *
' * and publisher shall not be liable in any event for incidental or       *
' * consequential damages in connection with, or arising out of, the       *
' * furnishing, performance, or use of these programs.                     *
' **************************************************************************