' Fig. 9.3: Time.vb
' Represents time and contains overloaded constructors.
Class Time
   ' declare Integer instance variables for the hour, minute and second
   Private hourValue As Integer ' 0 - 23
   Private minuteValue As Integer ' 0 - 59
   Private secondValue As Integer ' 0 - 59

   ' constructor initializes hour to 12, minute to 0 and second to 0
   ' to ensure that each Time object starts in a consistent state
   Public Sub New() ' parameterless constructor
      SetTime(12) ' initialize hour to noon; minute and second to 0
   End Sub ' New

   ' Time constructor: hour supplied;
   ' minute and second default to 0
   Public Sub New(ByVal hh As Integer)
      SetTime(hh) ' call SetTime with one argument
   End Sub ' New

   ' Time constructor: hour and minute supplied; 
   ' second defaults to 0
   Public Sub New(ByVal hh As Integer, ByVal mm As Integer)
      SetTime(hh, mm) ' call SetTime with two arguments
   End Sub ' New

   ' Time constructor: hour, minute and second supplied
   Public Sub New(ByVal hh As Integer, _
      ByVal mm As Integer, ByVal ss As Integer)
      SetTime(hh, mm, ss) ' call SetTime with three arguments
   End Sub ' New

   ' Time constructor: another Time object supplied
   Public Sub New(ByVal tt As Time)
      SetTime(tt.Hour, tt.Minute, tt.Second)
   End Sub ' New

   ' set a new time value using universal time, check validity of the
   ' data, set invalid hour to 12, set invalid minute/second to zero
   Public Sub SetTime(Optional ByVal hh As Integer = 12, _
      Optional ByVal mm As Integer = 0, Optional ByVal ss As Integer = 0)
      Hour = hh ' set hourValue using the Hour property    
      Minute = mm ' set minuteValue using the Minute property
      Second = ss ' set secondValue using the Second property
   End Sub ' SetTime

   ' property Hour                                              
   Public Property Hour() As Integer
      Get ' return hourValue                                    
         Return hourValue
      End Get

      Set(ByVal value As Integer) ' set hourValue
         If (value >= 0 AndAlso value < 24) Then ' in range 0-23
            hourValue = value ' value is valid
         Else ' value is invalid
            hourValue = 12 ' set to default of noon
         End If
      End Set
   End Property ' Hour

   ' property Minute                                            
   Public Property Minute() As Integer
      Get ' return minuteValue                                  
         Return minuteValue
      End Get

      Set(ByVal value As Integer) ' set minuteValue
         If (value >= 0 AndAlso value < 60) Then ' in range 0-59
            minuteValue = value ' value is valid
         Else ' value is invalid
            minuteValue = 0 ' set to default of 0
         End If
      End Set
   End Property ' Minute                                        

   ' property Second                                            
   Public Property Second() As Integer
      Get ' return secondValue                                  
         Return secondValue
      End Get

      Set(ByVal value As Integer) ' set secondValue
         If (value >= 0 AndAlso value < 60) Then ' in range 0-59
            secondValue = value ' value is valid
         Else ' value is invalid
            secondValue = 0 ' set to default of 0
         End If
      End Set
   End Property ' Second                                      

   ' convert Time to a String in universal-time (24-hour clock) format
   Public Function ToUniversalString() As String
      Return String.Format("{0}:{1:D2}:{2:D2}", Hour, Minute, Second)
   End Function ' ToUniversalString

   ' convert Time to a String in standard-time (12-hour clock) format
   Public Overrides Function ToString() As String
      Dim suffix As String ' AM or PM suffix
      Dim standardHour As Integer ' a standard hour in the range 1-12

      ' determine whether the 12-hour clock suffix should be AM or PM
      If Hour < 12 Then
         suffix = " AM" ' note space preceding AM
      Else
         suffix = " PM" ' note space preceding PM
      End If

      ' convert hour from universal-time format to standard-time format
      If (Hour = 12 OrElse Hour = 0) Then
         standardHour = 12
      Else
         standardHour = Hour Mod 12 ' 1 through 11, AM or PM
      End If

      Return String.Format("{0}:{1:D2}:{2:D2}", standardHour, Minute, _
         Second) & suffix
   End Function ' ToString
End Class ' Time



' **************************************************************************
' * (C) Copyright 1992-2006 by Deitel & Associates, Inc. and               *
' * Pearson Education, Inc. All Rights Reserved.                           *
' *                                                                        *
' * DISCLAIMER: The authors and publisher of this book have used their     *
' * best efforts in preparing the book. These efforts include the          *
' * development, research, and testing of the theories and programs        *
' * to determine their effectiveness. The authors and publisher make       *
' * no warranty of any kind, expressed or implied, with regard to these    *
' * programs or to the documentation contained in these books. The authors *
' * and publisher shall not be liable in any event for incidental or       *
' * consequential damages in connection with, or arising out of, the       *
' * furnishing, performance, or use of these programs.                     *
' **************************************************************************