' Fig. 14.28: FrmTreeViewDirectoryStructure.vb
' Using TreeView to display directory structure.
Imports System.IO

Public Class FrmTreeViewDirectoryStructure
   ' clear treDirectory, then call PopulateTreeView
   Private Sub btnEnter_Click(ByVal sender As System.Object, _
      ByVal e As System.EventArgs) Handles btnEnter.Click

      treDirectory.Nodes.Clear() ' clear all nodes

      ' if the directory specified by the user exists, fill in the 
      ' TreeView; otherwise, display an error message
      If (Directory.Exists(txtInput.Text)) Then
         ' add full path name to directoryTreeView 
         treDirectory.Nodes.Add(txtInput.Text)

         ' insert subfolders
         PopulateTreeView(txtInput.Text, treDirectory.Nodes(0))
      Else ' display error MessageBox if directory not found
         MessageBox.Show(txtInput.Text & " could not be found.", _
            "Directory Not Found", MessageBoxButtons.OK, _
            MessageBoxIcon.Error)
      End If
   End Sub ' btnEnter

   ' populate current node with subdirectories
   Private Sub PopulateTreeView( _
      ByVal directoryValue As String, ByVal parentNode As TreeNode)
      ' array stores all subdirectories in the directory
      Dim directoryArray As String() = _
         Directory.GetDirectories(directoryValue)

      ' populate current node with subdirectories
      Try
         ' check to see if any subdirectories are present
         If directoryArray.Length <> 0 Then
            ' for every subdirectory, create new TreeNode,
            ' add as a child of current node and recursively
            ' populate child nodes with subdirectories
            For Each directory As String In directoryArray
               ' obtain last part of path name from the full path name
               ' by finding the last occurence of "\" and returning the
               ' part of the path name that comes after this occurrence
               Dim directoryName As String = directory.Substring( _
                  directory.LastIndexOf("\") + 1)

               ' create TreeNode for current directory
               Dim myNode As TreeNode = New TreeNode(directoryName)

               ' add current directory node to parent node
               parentNode.Nodes.Add(myNode)

               ' recursively populate every subdirectory
               PopulateTreeView(directory, myNode)
            Next
         End If
      Catch e As UnauthorizedAccessException
         parentNode.Nodes.Add("Access denied")
      End Try
   End Sub ' PopulateTreeView
End Class ' FrmTreeViewDirectoryStructure


' **************************************************************************
' * (C) Copyright 1992-2006 by Deitel & Associates, Inc. and               *
' * Pearson Education, Inc. All Rights Reserved.                           *
' *                                                                        *
' * DISCLAIMER: The authors and publisher of this book have used their     *
' * best efforts in preparing the book. These efforts include the          *
' * development, research, and testing of the theories and programs        *
' * to determine their effectiveness. The authors and publisher make       *
' * no warranty of any kind, expressed or implied, with regard to these    *
' * programs or to the documentation contained in these books. The authors *
' * and publisher shall not be liable in any event for incidental or       *
' * consequential damages in connection with, or arising out of, the       *
' * furnishing, performance, or use of these programs.                     *
' **************************************************************************

