' Fig. 15.11: CircularBuffer.vb
' A circular shared buffer for the producer/consumer relationship.
Imports System.Threading

' implement the an array of shared integers with synchronization
Public Class CircularBuffer
   Implements IBuffer
   ' each array element is a buffer
   Private buffers As Integer() = {-1, -1, -1}

   ' occupiedBufferCount maintains count of occupied buffers
   Private occupiedBufferCount As Integer = 0

   ' variable that maintain read and write buffer locations
   Private readLocation As Integer = 0
   Private writeLocation As Integer = 0

   ' property Buffer
   Public Property Buffer() As Integer Implements IBuffer.Buffer
      Get
         ' lock this object while getting value from buffers array
         SyncLock (Me)
            ' if there is no data to read, place invoking thread 
            ' in WaitSleepJoin state
            If occupiedBufferCount = 0 Then
               Console.Write(vbCrLf & "All buffers empty. " & _
                  Thread.CurrentThread.Name & " waits.")
               Monitor.Wait(Me) ' enter the WaitSleepJoin state
            End If

            ' obtain value at current readLocation
            Dim readValue As Integer = buffers(readLocation)

            Console.Write(vbCrLf & Thread.CurrentThread.Name & _
                " reads " & buffers(readLocation) & " ")

            ' just consumed a value, so decrement 
            ' number of occupied buffers
            occupiedBufferCount -= 1

            ' update readLocation for future read operation, then 
            ' add current state to output
            readLocation = (readLocation + 1) Mod buffers.Length
            CreateStateOutput()

            ' return waiting thread (if there is one) to Running state
            Monitor.Pulse(Me)

            Return readValue
         End SyncLock
      End Get
      Set(ByVal value As Integer)
         ' lock this object while setting value in buffers array
         SyncLock (Me)
            ' if there are no empty locations, place invoking thread 
            ' in WaitSleepJoin state
            If occupiedBufferCount = buffers.Length Then
               Console.Write(vbCrLf & "All buffers full. " & _
                  Thread.CurrentThread.Name & " waits.")
               Monitor.Wait(Me) ' enter the WaitSleepJoin state
            End If

            ' place value in writeLocation of buffers
            buffers(writeLocation) = value

            Console.Write(vbCrLf & Thread.CurrentThread.Name & _
               " writes " & buffers(writeLocation) & " ")

            ' just produced a value, so increment 
            ' number of occupied buffers
            occupiedBufferCount += 1

            ' update writeLocation for future write operation, 
            ' then add current state to output
            writeLocation = (writeLocation + 1) Mod buffers.Length
            CreateStateOutput()

            ' return waiting thread (if there is one) to Running state
            Monitor.Pulse(Me)
         End SyncLock
      End Set
   End Property ' Buffer

   ' create state output
   Public Sub CreateStateOutput()
      ' display first line of state information
      Console.Write("(buffers occupied: " & _
         occupiedBufferCount & ")" & vbCrLf & "buffers: ")

      For i As Integer = 0 To (buffers.Length - 1)
         Console.Write(" " & String.Format("{0,2}", buffers(i)) & "  ")
      Next (i)

      Console.WriteLine()

      ' display second line of state information
      Console.Write("         ")

      For i As Integer = 0 To (buffers.Length - 1)
         Console.Write("---- ")
      Next (i)

      Console.WriteLine()

      ' display third line of state information
      Console.Write("         ")

      ' display readLocation (R) and writeLocation (W) indicators 
      ' below appropriate buffer locations
      For i As Integer = 0 To (buffers.Length - 1)
         If i = writeLocation And writeLocation = readLocation Then
            Console.Write(" WR  ")
         ElseIf i = writeLocation Then
            Console.Write(" W   ")
         ElseIf i = readLocation Then
            Console.Write("  R  ")
         Else
            Console.Write("     ")
         End If
      Next (i)

      Console.WriteLine()
   End Sub ' CreateStateOutput
End Class ' CircularBuffer

'**************************************************************************
'* (C) Copyright 1992-2006 by Deitel & Associates, Inc. and               *
'* Pearson Education, Inc. All Rights Reserved.                           *
'*                                                                        *
'* DISCLAIMER: The authors and publisher of this book have used their     *
'* best efforts in preparing the book. These efforts include the          *
'* development, research, and testing of the theories and programs        *
'* to determine their effectiveness. The authors and publisher make       *
'* no warranty of any kind, expressed or implied, with regard to these    *
'* programs or to the documentation contained in these books. The authors *
'* and publisher shall not be liable in any event for incidental or       *
'* consequential damages in connection with, or arising out of, the       *
'* furnishing, performance, or use of these programs.                     *
'**************************************************************************