' Fig 18.6: FrmFileSearch.vb
' Using regular expressions to determine file types.
Imports System.IO
Imports System.Text.RegularExpressions
Imports System.Collections.Specialized

Public Class FrmFileSearch
   Dim currentDirectory As String = Directory.GetCurrentDirectory()
   Dim directoryList() As String ' subdirectories
   Dim fileArray() As String

   ' store extensions found and number found
   Dim found As New NameValueCollection()

   ' invoked when user types in text box
   Private Sub txtInput_KeyDown(ByVal sender As System.Object, _
      ByVal e As System.Windows.Forms.KeyEventArgs) _
      Handles txtInput.KeyDown
      ' determine whether user pressed Enter
      If e.KeyCode = Keys.Enter Then
         btnSearch_Click(sender, e)
      End If
   End Sub ' txtInput_KeyDown

   ' invoked when user clicks "Search Directory" button
   Private Sub btnSearch_Click(ByVal sender As System.Object, _
      ByVal e As System.EventArgs) Handles btnSearch.Click
      ' check for user input; default is current directory
      If txtInput.Text <> "" Then
         ' verify that user input is valid directory name
         If Directory.Exists(txtInput.Text) Then
            currentDirectory = txtInput.Text

            ' reset input text box and update display
            lblDirectory.Text = _
               "Current Directory:" & vbCrLf & currentDirectory
         Else
            ' show error if user does not specify valid directory
            MessageBox.Show("Invalid Directory", "Error", _
               MessageBoxButtons.OK, MessageBoxIcon.Error)
         End If
      End If

      ' clear text boxes
      txtInput.Text = ""
      txtOutput.Text = ""

      SearchDirectory(currentDirectory) ' search directory

      ' summarize and print results
      For Each current As String In found
         txtOutput.Text &= "* Found " & found(current) & _
            " " & current & " files." & vbCrLf
      Next current

      ' clear output for new search
      found.Clear()
   End Sub ' btnSearch_Click

   ' search directory using regular expression
   Private Sub SearchDirectory(ByVal currentDirectory As String)
      ' for file name without directory path
      Try
         Dim fileName As String = ""

         ' regular expression for extensions matching pattern
         Dim regularExpression As New Regex( _
            "[a-zA-Z0-9]+\.(?<extension>\w+)")

         ' stores regular-expression match result
         Dim matchResult As Match

         Dim fileExtension As String ' holds file extensions

         ' number of files with given extension in directory
         Dim extensionCount As Integer

         ' get directories
         directoryList = Directory.GetDirectories(currentDirectory)

         ' get list of files in current directory
         fileArray = Directory.GetFiles(currentDirectory)

         ' iterate through list of files
         For Each myFile As String In fileArray
            ' remove directory path from file name
            fileName = myFile.Substring(myFile.LastIndexOf("\") + 1)

            ' obtain result for regular-expression search
            matchResult = regularExpression.Match(fileName)

            ' check for match
            If matchResult.Success Then
               fileExtension = matchResult.Result("${extension}")
            Else
               fileExtension = "[no extension]"
            End If

            ' store value from container
            If found(fileExtension) Is Nothing Then
               found.Add(fileExtension, "1")
            Else
               extensionCount = Int32.Parse(found(fileExtension)) + 1
               found(fileExtension) = extensionCount.ToString()
            End If

            ' search for backup( .bak ) files
            If fileExtension = "bak" Then
               ' prompt user to delete ( .bak ) file
               Dim result As DialogResult = MessageBox.Show( _
                  "Found backup file " & fileName & ". Delete?", _
                  "Delete Backup", MessageBoxButtons.YesNo, _
                  MessageBoxIcon.Question)

               ' delete file if user clicked 'yes'
               If result = Windows.Forms.DialogResult.Yes Then
                  File.Delete(myFile)
                  extensionCount = Int32.Parse(found("bak")) - 1
                  found("bak") = extensionCount.ToString()
               End If
            End If
         Next myFile

         ' recursive call to search files in subdirectory
         For Each myDirectory As String In directoryList
            SearchDirectory(myDirectory)
         Next myDirectory
         ' handle exception if files have unauthorized access
      Catch ex As UnauthorizedAccessException
         MessageBox.Show("Some files may not be visible" & _
            " due to permission settings", "Warning", _
            MessageBoxButtons.OK, MessageBoxIcon.Information)
      End Try
   End Sub ' SearchDirectory
End Class ' FrmFileSearch


'**************************************************************************
'* (C) Copyright 1992-2006 by Deitel & Associates, Inc. and               *
'* Pearson Education, Inc. All Rights Reserved.                           *
'*                                                                        *
'* DISCLAIMER: The authors and publisher of this book have used their     *
'* best efforts in preparing the book. These efforts include the          *
'* development, research, and testing of the theories and programs        *
'* to determine their effectiveness. The authors and publisher make       *
'* no warranty of any kind, expressed or implied, with regard to these    *
'* programs or to the documentation contained in these books. The authors *
'* and publisher shall not be liable in any event for incidental or       *
'* consequential damages in connection with, or arising out of, the       *
'* furnishing, performance, or use of these programs.                     *
'**************************************************************************