' Fig 18.14: FrmCreateFile.vb
' Creating a sequential-access file using serialization.
Imports System.IO
Imports System.Runtime.Serialization.Formatters.Binary
Imports System.Runtime.Serialization
Imports BankLibrary

Public Class FrmCreateFile
   ' object for serializing Records in binary format
   Private formatter As New BinaryFormatter()
   Private output As FileStream ' stream for writing to a file

   ' handler for btnSave_Click
   Private Sub btnSave_Click(ByVal sender As System.Object, _
      ByVal e As System.EventArgs) Handles btnSave.Click
      ' create dialog box enabling user to save file
      Dim fileChooser As New SaveFileDialog()
      Dim result As DialogResult = fileChooser.ShowDialog()
      Dim fileName As String ' name of file to save data

      fileChooser.CheckFileExists = False ' allow user to create file

      ' exit event handler if user clicked "Cancel"
      If result = Windows.Forms.DialogResult.Cancel Then
         Return
      End If

      fileName = fileChooser.FileName ' get specified file name

      ' show error if user specified invalid file
      If fileName = "" Or fileName Is Nothing Then
         MessageBox.Show("Invalid File Name", "Error", _
            MessageBoxButtons.OK, MessageBoxIcon.Error)
      Else
         ' save file via FileStream if user specified valid file
         Try
            ' open file with write access
            output = New FileStream( _
               fileName, FileMode.OpenOrCreate, FileAccess.Write)

            ' disable Save button and enable Enter button
            btnSave.Enabled = False
            btnEnter.Enabled = True
            ' handle exception if there is a problem opening the file
         Catch ex As IOException
            ' notify user if file does not exist
            MessageBox.Show("Error opening file", "Error", _
               MessageBoxButtons.OK, MessageBoxIcon.Error)
         End Try
      End If
   End Sub ' btnSave_Click

   ' handler for btnEnter_Click
   Private Sub btnEnter_Click(ByVal sender As System.Object, _
      ByVal e As System.EventArgs) Handles btnEnter.Click
      ' store TextBox values string array
      Dim values As String() = GetTextBoxValues()

      ' Record containing TextBox values to serialize
      Dim record As New RecordSerializable()

      ' determine whether TextBox account field is empty
      If values(TextBoxIndices.ACCOUNT) <> "" Then
         ' store TextBox values in Record and serialize Record
         Try
            ' get account number value from TextBox
            Dim accountNumber As Integer = _
               Int32.Parse(values(TextBoxIndices.ACCOUNT))

            ' determine whether accountNumber is valid
            If accountNumber > 0 Then
               ' store TextBox fields in Record
               record.Account = accountNumber
               record.FirstName = _
                  values(TextBoxIndices.FIRST)
               record.LastName = _
                  values(TextBoxIndices.LAST)
               record.Balance = _
                  Decimal.Parse(values(TextBoxIndices.BALANCE))

               ' write Record to FileStream ( serialize object )
               formatter.Serialize(output, record)
            Else
               ' notify user if invalid account number
               MessageBox.Show("Invalid Account Number", "Error", _
                  MessageBoxButtons.OK, MessageBoxIcon.Error)
            End If
            ' notify user if error occurs in serialization
         Catch ex As SerializationException
            MessageBox.Show("Error Writing to File", "Error", _
               MessageBoxButtons.OK, MessageBoxIcon.Error)
            ' notify user if error occurs regarding parameter format
         Catch ex As FormatException
            MessageBox.Show("Invalid Format", "Error", _
               MessageBoxButtons.OK, MessageBoxIcon.Error)
         End Try
      End If

      ClearTextBoxes() ' clear TextBox values
   End Sub ' btnEnter_Click

   ' handler for btnSave_Click
   Private Sub btnExit_Click(ByVal sender As System.Object, _
      ByVal e As System.EventArgs) Handles btnExit.Click
      ' determine whether file exists
      If output IsNot Nothing Then
         ' close file
         Try
            output.Close()
            ' notify user of error closing file
         Catch ex As IOException
            MessageBox.Show("Cannot close file", "Error", _
               MessageBoxButtons.OK, MessageBoxIcon.Error)
         End Try
      End If

      Application.Exit()
   End Sub ' btnEnter_Click
End Class ' FrmCreateFile


'**************************************************************************
'* (C) Copyright 1992-2006 by Deitel & Associates, Inc. and               *
'* Pearson Education, Inc. All Rights Reserved.                           *
'*                                                                        *
'* DISCLAIMER: The authors and publisher of this book have used their     *
'* best efforts in preparing the book. These efforts include the          *
'* development, research, and testing of the theories and programs        *
'* to determine their effectiveness. The authors and publisher make       *
'* no warranty of any kind, expressed or implied, with regard to these    *
'* programs or to the documentation contained in these books. The authors *
'* and publisher shall not be liable in any event for incidental or       *
'* consequential damages in connection with, or arising out of, the       *
'* furnishing, performance, or use of these programs.                     *
'**************************************************************************