' Fig. 19.25: FrmPathNavigator.vb
' Demonstrates class XPathNavigator.
Imports System.Xml.XPath

Public Class FrmPathNavigator
   Private xPath As XPathNavigator ' navigator to traverse document
   Private document As XPathDocument ' document for use by XPathNavigator
   Private tree As TreeNode ' TreeNode used by TreeView control

   ' initialize variables and TreeView control
   Private Sub FrmPathNavigator_Load(ByVal sender As Object, _
      ByVal e As EventArgs) Handles MyBase.Load
      document = New XPathDocument("sports.xml") ' load XML document
      xPath = document.CreateNavigator() ' create navigator
      tree = New TreeNode() ' create root node for TreeNodes

      tree.Text = xPath.NodeType.ToString() ' #root
      treePath.Nodes.Add(tree) ' add tree

      ' update TreeView control
      treePath.ExpandAll() ' expand tree node in TreeView
      treePath.Refresh() ' force TreeView update
      treePath.SelectedNode = tree ' highlight root
   End Sub ' FrmPathNavigator_Load

   ' process btnSelect_Click event
   Private Sub btnSelect_Click(ByVal sender As Object, _
      ByVal e As EventArgs) Handles btnSelect.Click
      Dim iterator As XPathNodeIterator ' enables node iteration

      Try ' get specified node from ComboBox
         iterator = xPath.Select(cboSelect.Text) ' select specified node
         DisplayIterator(iterator) ' display selection
      Catch argumentException As XPathException
         MessageBox.Show(argumentException.Message, "Error", _
            MessageBoxButtons.OK, MessageBoxIcon.Error)
      End Try
   End Sub ' btnSelect_Click

   ' traverse to first child on btnFirstChild_Click event
   Private Sub btnFirstChild_Click(ByVal sender As Object, _
      ByVal e As EventArgs) Handles btnFirstChild.Click
      Dim newTreeNode As TreeNode

      ' move to first child
      If xPath.MoveToFirstChild() Then
         newTreeNode = New TreeNode() ' create new node

         ' set node's Text property to either navigator's name or value
         DetermineType(newTreeNode, xPath)
         tree.Nodes.Add(newTreeNode) ' add nodes to TreeNode node list
         tree = newTreeNode ' assign tree newTreeNode

         ' update TreeView control
         treePath.ExpandAll() ' expand node in TreeView
         treePath.Refresh() ' force TreeView to update
         treePath.SelectedNode = tree ' highlight root
      Else ' node has no children
         MessageBox.Show("Current Node has no children.", _
               "", MessageBoxButtons.OK, MessageBoxIcon.Information)
      End If
   End Sub ' btnFirstChild_Click

   ' traverse to node's parent on btnParent_Click event
   Private Sub btnParent_Click(ByVal sender As Object, _
      ByVal e As EventArgs) Handles btnParent.Click
      ' move to parent
      If xPath.MoveToParent() Then
         tree = tree.Parent

         ' get number of child nodes, not including sub trees
         Dim count As Integer = tree.GetNodeCount(False)

         ' remove all children
         For i As Integer = 0 To count - 1
            tree.Nodes.Remove(tree.FirstNode) ' remove child node
         Next

         ' update TreeView control
         treePath.ExpandAll() ' expand node in TreeView
         treePath.Refresh() ' force TreeView to update
         treePath.SelectedNode = tree ' highlight root
      Else ' if node has no parent (root node)
         MessageBox.Show("Current node has no parent.", "", _
            MessageBoxButtons.OK, MessageBoxIcon.Information)
      End If
   End Sub ' btnParent_Click

   ' find next sibling on btnNext_Click event
   Private Sub btnNext_Click(ByVal sender As Object, _
      ByVal e As EventArgs) Handles btnNext.Click
      ' declare and initialize two TreeNodes
      Dim newTreeNode As TreeNode = Nothing
      Dim newNode As TreeNode = Nothing

      ' move to next sibling
      If xPath.MoveToNext() Then
         newTreeNode = tree.Parent ' get parent node
         newNode = New TreeNode() ' create new node

         ' decide whether to display current node
         DetermineType(newNode, xPath)
         newTreeNode.Nodes.Add(newNode) ' add to parent node

         tree = newNode ' set current position for display

         ' update TreeView control
         treePath.ExpandAll() ' expand node in TreeView
         treePath.Refresh() ' force TreeView to update
         treePath.SelectedNode = tree ' highlight root
      Else ' node has no additional siblings
         MessageBox.Show("Current node is last sibling.", "", _
            MessageBoxButtons.OK, MessageBoxIcon.Information)
      End If
   End Sub ' btnNext_Click

   ' get previous sibling on btnPrevious_Click
   Private Sub btnPrevious_Click(ByVal sender As Object, _
      ByVal e As EventArgs) Handles btnPrevious.Click
      Dim parentTreeNode As TreeNode = Nothing

      ' move to previous sibling
      If xPath.MoveToPrevious() Then

         parentTreeNode = tree.Parent ' get parent node
         parentTreeNode.Nodes.Remove(tree) ' delete current node
         tree = parentTreeNode.LastNode ' move to previous node

         ' update TreeView control
         treePath.ExpandAll() ' expand tree node in TreeView
         treePath.Refresh() ' force TreeView to update
         treePath.SelectedNode = tree ' highlight root
      Else ' if current node has no previous siblings
         MessageBox.Show("Current node is first sibling.", "", _
            MessageBoxButtons.OK, MessageBoxIcon.Information)
      End If
   End Sub ' btnPrevious_Click

   ' print values for XPathNodeIterator
   Private Sub DisplayIterator(ByVal iterator As XPathNodeIterator)
      txtSelect.Clear()

      ' display selected node's values
      While iterator.MoveNext()
         txtSelect.Text &= iterator.Current.Value.Trim() & vbCrLf
      End While
   End Sub ' DisplayIterator

   ' determine if TreeNode should display current node name or value
   Private Sub DetermineType(ByVal node As TreeNode, _
      ByVal xPath As XPathNavigator)

      Select Case xPath.NodeType ' determine NodeType
         Case XPathNodeType.Element ' if Element, get its name
            ' get current node name, and remove whitespaces
            node.Text = xPath.Name.Trim()
         Case Else ' obtain node values
            ' get current node value and remove whitespaces
            node.Text = xPath.Value.Trim()
      End Select
   End Sub ' DetermineType
End Class ' FrmPathNavigator

' **************************************************************************
' * (C) Copyright 1992-2006 by Deitel & Associates, Inc. and               *
' * Pearson Education, Inc. All Rights Reserved.                           *
' *                                                                        *
' * DISCLAIMER: The authors and publisher of this book have used their     *
' * best efforts in preparing the book. These efforts include the          *
' * development, research, and testing of the theories and programs        *
' * to determine their effectiveness. The authors and publisher make       *
' * no warranty of any kind, expressed or implied, with regard to these    *
' * programs or to the documentation contained in these books. The authors *
' * and publisher shall not be liable in any event for incidental or       *
' * consequential damages in connection with, or arising out of, the       *
' * furnishing, performance, or use of these programs.                     *
' **************************************************************************