' Fig. 22.18: BlackjackWebService.vb
' Blackjack Web Service deals and counts cards.
Imports System.Web
Imports System.Web.Services
Imports System.Web.Services.Protocols

<WebService(Namespace:="http://www.deitel.com/", Description:= _
   "A Web service that deals and counts cards for the game Blackjack")> _
<WebServiceBinding(ConformsTo:=WsiProfiles.BasicProfile1_1)> _
<Global.Microsoft.VisualBasic.CompilerServices.DesignerGenerated()> _
Public Class BlackjackWebService
   Inherits System.Web.Services.WebService

   ' deals card that has not yet been dealt
   <WebMethod(EnableSession:=True, _
      Description:="Deal a new card from the deck.")> _
   Public Function DealCard() As String
      ' get client's deck
      Dim deck As ArrayList = CType(Session("deck"), ArrayList)
      Dim card As String = Convert.ToString(deck(0))
      deck.RemoveAt(0)
      Return card
   End Function ' DealCard

   ' creates and shuffles a deck of cards
   <WebMethod(EnableSession:=True, _
      Description:="Create and shuffle a deck of cards.")> _
   Public Sub Shuffle()
      Dim temporary As Object ' holds card temporarily during swapping
      Dim randomObject As New Random() ' generates random numbers
      Dim newIndex As Integer ' index of randomly selected card
      Dim deck As New ArrayList() ' stores deck of cards (strings)

      ' generate all possible cards
      For face As Integer = 1 To 13 ' loop through face values
         For suit As Integer = 0 To 3 ' loop through suits
            deck.Add(face & " " & suit) ' add card (string) to deck
         Next
      Next

      ' shuffles deck by swapping each card with another card randomly
      For i As Integer = 0 To deck.Count - 1
         ' get random index
         newIndex = randomObject.Next(deck.Count - 1)
         temporary = deck(i) ' save current card in temporary variable
         deck(i) = deck(newIndex) ' copy randomly selected card
         deck(newIndex) = temporary ' copy current card back into deck
      Next

      Session.Add("deck", deck) ' add this deck to user's session state
   End Sub ' Shuffle

   ' computes value of hand
   <WebMethod(Description:= _
      "Compute a numerical value for the current hand.")> _
   Public Function GetHandValue(ByVal dealt As String) As Integer
      ' split string containing all cards
      Dim tab As Char() = New Char() {Convert.ToChar(vbTab)}
      Dim cards As String() = dealt.Split(tab) ' get array of cards
      Dim total As Integer = 0 ' total value of cards in hand
      Dim face as integer ' face of the current card
      Dim aceCount As Integer = 0 ' number of aces in hand

      ' loop through the cards in the hand
      For Each drawn As String In cards
         ' get face of card
         face = Int32.Parse(drawn.Substring(0, drawn.IndexOf(" ")))

         Select Case face
            Case 1 ' if ace, increment aceCount
               aceCount += 1
            Case 11 To 13 ' if jack, queen or king add 10
               total += 10
            Case Else ' otherwise, add value of face 
               total += face
         End Select
      Next

      ' if there are any aces, calculate optimum total
      If aceCount > 0 Then
         ' if it is possible to count one ace as 11, and the rest
         ' as 1 each, do so otherwise, count all aces as 1 each
         If (total + 11 + aceCount - 1 <= 21) Then
            total += 11 + aceCount - 1
         Else
            total += aceCount
         End If
      End If

      Return total
   End Function ' GetHandValue
End Class ' BlackjackWebService

'**************************************************************************
'* (C) Copyright 1992-2006 by Deitel & Associates, Inc. and               *
'* Pearson Education, Inc. All Rights Reserved.                           *
'*                                                                        *
'* DISCLAIMER: The authors and publisher of this book have used their     *
'* best efforts in preparing the book. These efforts include the          *
'* development, research, and testing of the theories and programs        *
'* to determine their effectiveness. The authors and publisher make       *
'* no warranty of any kind, expressed or implied, with regard to these    *
'* programs or to the documentation contained in these books. The authors *
'* and publisher shall not be liable in any event for incidental or       *
'* consequential damages in connection with, or arising out of, the       *
'* furnishing, performance, or use of these programs.                     *
'**************************************************************************
