' Fig. 23.1: FrmChatServer.vb
' Set up a server that will receive a connection from a client, send a
' string to the client, chat with the client and close the connection.
Imports System.Threading
Imports System.Net
Imports System.Net.Sockets
Imports System.IO

Public Class FrmChatServer
   Private connection As Socket ' Socket for accepting a connection      
   Private readThread As Thread ' Thread for processing incoming messages
   Private socketStream As NetworkStream ' network data stream           
   Private writer As BinaryWriter ' facilitates writing to the stream    
   Private reader As BinaryReader ' facilitates reading from the stream

   ' initialize thread for reading
   Private Sub FrmChatServer_Load(ByVal sender As System.Object, _
      ByVal e As System.EventArgs) Handles MyBase.Load

      readThread = New Thread(New ThreadStart(AddressOf RunServer))
      readThread.Start()
   End Sub ' FrmChatServer_Load

   ' close all threads associated with this application
   Private Sub FrmChatServer_FormClosing(ByVal sender As System.Object, _
      ByVal e As System.Windows.Forms.FormClosingEventArgs) _
      Handles MyBase.FormClosing

      System.Environment.Exit(System.Environment.ExitCode)
   End Sub ' FrmChatServer_FormClosing

   ' Delegate that allows method DisplayMessage to be called
   ' in the thread that creates and maintains the GUI       
   Private Delegate Sub DisplayDelegate(ByVal message As String)

   ' method DisplayMessage sets txtDisplay's Text property
   ' in a thread-safe manner
   Private Sub DisplayMessage(ByVal message As String)
      ' if modifying txtDisplay is not thread safe
      If txtDisplay.InvokeRequired Then
         ' use inherited method Invoke to execute DisplayMessage
         ' via a Delegate                                       
         Invoke(New DisplayDelegate(AddressOf DisplayMessage), _
            New Object() {message})
         ' OK to modify txtDisplay in current thread
      Else
         txtDisplay.Text &= message
      End If
   End Sub ' DisplayMessage

   ' Delegate that allows method DisableInput to be called 
   ' in the thread that creates and maintains the GUI
   Private Delegate Sub DisableInputDelegate(ByVal value As Boolean)

   ' method DisableInput sets txtInput's ReadOnly property
   ' in a thread-safe manner
   Private Sub DisableInput(ByVal value As Boolean)
      ' if modifying txtInput is not thread safe
      If txtInput.InvokeRequired Then
         ' use inherited method Invoke to execute DisableInput
         ' via a Delegate                                     
         Invoke(New DisableInputDelegate(AddressOf DisableInput), _
            New Object() {value})
         ' OK to modify txtInput in current thread
      Else
         txtInput.ReadOnly = value
      End If
   End Sub ' DisableInput

   ' send the text typed at the server to the client
   Private Sub txtInput_KeyDown(ByVal sender As System.Object, _
      ByVal e As System.Windows.Forms.KeyEventArgs) _
      Handles txtInput.KeyDown
      ' send the text to the client
      Try
         If e.KeyCode = Keys.Enter And txtInput.ReadOnly = False Then
            writer.Write("SERVER>>> " & txtInput.Text)
            txtDisplay.Text &= vbCrLf & "SERVER>>> " & txtInput.Text

            ' if the user at the server signaled termination
            ' sever the connection to the client
            If txtInput.Text = "TERMINATE" Then
               connection.Close()
            End If
            txtInput.Clear() ' clear the users input
         End If
      Catch ex As SocketException
         txtDisplay.Text &= vbCrLf & "Error writing object"
      End Try
   End Sub ' txtInput_KeyDown

   ' allows a client to connect; displays text the client sends
   Public Sub RunServer()
      Dim listener As TcpListener
      Dim counter As Integer = 1

      ' wait for a client connection and display the text
      ' that the client sends
      Try
         ' Step 1: create TcpListener                    
         Dim local As IPAddress = IPAddress.Parse("127.0.0.1")
         listener = New TcpListener(local, 50000)

         ' Step 2: TcpListener waits for connection request
         listener.Start()

         ' Step 3: establish connection upon client request
         While True
            DisplayMessage("Waiting for connection" & vbCrLf)

            ' accept an incoming connection     
            connection = listener.AcceptSocket()

            ' create NetworkStream object associated with socket
            socketStream = New NetworkStream(connection)

            ' create objects for transferring data across stream
            writer = New BinaryWriter(socketStream)
            reader = New BinaryReader(socketStream)

            DisplayMessage( _
               "Connection " & counter & " received." & vbCrLf)

            ' inform client that connection was successfull  
            writer.Write("SERVER>>> Connection successful")

            DisableInput(False) ' enable txtInput
            Dim theReply As String = ""

            ' Step 4: read string data sent from client
            Do
               Try
                  ' read the string sent to the server
                  theReply = reader.ReadString()

                  ' display the message
                  DisplayMessage(vbCrLf & theReply)
               Catch ex As Exception
                  ' handle exception if error reading data
                  Exit Do
               End Try
            Loop While theReply <> "CLIENT>>> TERMINATE" And _
               connection.Connected

            DisplayMessage(vbCrLf & "User terminated connection" & vbCrLf)

            ' Step 5: close connection  
            writer.Close()
            reader.Close()
            socketStream.Close()
            connection.Close()

            DisableInput(True) ' disable txtInput
            counter += 1
         End While
      Catch ex As Exception
         MessageBox.Show(ex.ToString())
      End Try
   End Sub ' RunServer
End Class ' FrmChatServer


'**************************************************************************
'* (C) Copyright 1992-2006 by Deitel & Associates, Inc. and               *
'* Pearson Education, Inc. All Rights Reserved.                           *
'*                                                                        *
'* DISCLAIMER: The authors and publisher of this book have used their     *
'* best efforts in preparing the book. These efforts include the          *
'* development, research, and testing of the theories and programs        *
'* to determine their effectiveness. The authors and publisher make       *
'* no warranty of any kind, expressed or implied, with regard to these    *
'* programs or to the documentation contained in these books. The authors *
'* and publisher shall not be liable in any event for incidental or       *
'* consequential damages in connection with, or arising out of, the       *
'* furnishing, performance, or use of these programs.                     *
'**************************************************************************