' Fig. 23.11: ReportInfo.vb
' Class that implements interface Report, retrieves
' and returns data on weather
Imports System.Collections
Imports System.IO
Imports System.Net
Imports Weather

Public Class ReportInfo : Inherits MarshalByRefObject : Implements Report
   Private cityList As ArrayList ' cities, temperatures, descriptions

   Public Sub New()
      cityList = New ArrayList()

      ' create WebClient to get access to Web page
      Dim myClient As New WebClient()

      ' get StreamReader for response so we can read page     
      Dim input As New StreamReader(myClient.OpenRead( _
         "http://iwin.nws.noaa.gov/iwin/us/traveler.html"))

      ' indicates first batch of cities
      Dim separator1 As String = "TAV12"
      ' indicates second batch of cities
      Dim separator2 As String = "TAV13"

      ' locate separator1 in Web page                                   
      While Not input.ReadLine().StartsWith(separator1) ' do nothing
      End While
      ReadCities(input) ' read the first batch of cities

      ' locate separator2 in Web page                                   
      While Not input.ReadLine().StartsWith(separator2) ' do nothing
      End While
      ReadCities(input) ' read the second batch of cities

      cityList.Sort() ' sort list of cities by alphabetical order                         
      input.Close() ' close StreamReader to NWS server

      ' display the data on the server side
      Console.WriteLine("Data from NWS Web site:")

      For Each city As CityWeather In cityList
         Console.WriteLine(city)
      Next city ' end foreach
   End Sub ' New

   ' utility method that reads a batch of cities
   Private Sub ReadCities(ByVal input As StreamReader)
      ' day format and night format
      Dim dayFormat As String = _
         "CITY            WEA     HI/LO   WEA     HI/LO"
      Dim nightFormat As String = _
         "CITY            WEA     LO/HI   WEA     LO/HI"
      Dim inputLine As String = ""

      ' locate header that begins weather information
      Do
         inputLine = input.ReadLine()
      Loop While Not inputLine.Equals(dayFormat) And Not _
         inputLine.Equals(nightFormat)

      inputLine = input.ReadLine() ' get first city's data

      ' while there are more cities to read
      While inputLine.Length > 28
         ' create CityWeather object for city 
         Dim weather As New CityWeather(inputLine.Substring(0, 16), _
            inputLine.Substring(16, 7), inputLine.Substring(23, 7))

         cityList.Add(weather) ' add to ArrayList         
         inputLine = input.ReadLine() ' get next city's data
      End While ' end while
   End Sub ' ReadCities

   ' property for getting the cities' weather reports
   Public ReadOnly Property Reports() As _
      System.Collections.ArrayList Implements Weather.Report.Reports

      Get
         Return cityList
      End Get
   End Property ' Reports
End Class ' ReportInfo

'**************************************************************************
'* (C) Copyright 1992-2006 by Deitel & Associates, Inc. and               *
'* Pearson Education, Inc. All Rights Reserved.                           *
'*                                                                        *
'* DISCLAIMER: The authors and publisher of this book have used their     *
'* best efforts in preparing the book. These efforts include the          *
'* development, research, and testing of the theories and programs        *
'* to determine their effectiveness. The authors and publisher make       *
'* no warranty of any kind, expressed or implied, with regard to these    *
'* programs or to the documentation contained in these books. The authors *
'* and publisher shall not be liable in any event for incidental or       *
'* consequential damages in connection with, or arising out of, the       *
'* furnishing, performance, or use of these programs.                     *
'**************************************************************************
